/*******************************************

This is the "main" module for EPROM Programmer
software version 2.0.  The other source files are:
	input.cpp
	scrn_out.cpp
	utility.cpp
	prom_io.cpp
	t_loop.asm (assembly language routine
		for software timer)

Function and variable declarations are located in 
eprmdecs.h

The source code was written, compiled and linked with
Borland C++ version 4.0 using the medium memory model.

********************************************/



#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <string.h>
#include <dir.h>
#include <io.h>


/* Define constants for pre-processor */
#define CORNER_TL  0xc9
#define CORNER_TR  0xbb
#define CORNER_BL  0xc8
#define CORNER_BR  0xbc
#define HORIZ_LINE  0xcd
#define VERT_LINE  0xba
#define ALT_P 25
#define ALT_C 46
#define ALT_R 19
#define ALT_A 30
#define ALT_Q 16
#define ALWAYS 1
#define DOWNARROW 0x50
#define UPARROW 0x48
#define ESCAPE 0x1b
#define RETURN 0xd


/* FUNCTION DECLARATIONS */

/* Routines involved in actually reading and writing to PROMs:
located in file PROM_IO.CPP */
	void prom_write (void);
	unsigned char read_byte (void);
	void manual_read (void);
	void read_tofile (void);
	void compare (void);

/* Routines involved in fetching user input: located in file INPUT.CPP.
 The "get..." functions fetch user input for the relevant variables. The rest
 are generic routines used by the "get..." functions and others. */
	void getread_variables (void);
	void getcompare_variables (void);
	void getprogram_variables (int);
	void text_input (char *, int, int);
	unsigned long longint_input (char *, int, int);
	float float_input (char *, int, int);
	int input_yn (void);
	void custom_bytes (int);
	char input_0_1 (int);

/* Utility and miscellaneous functions: located in file UTILITY.CPP */
	void get_dftfile (void);
	void write_dftfile (void);
	void t_constant (void);
	int is_stringchar (char);
	int is_hexvalue (char);
	int is_digit (char);
	unsigned long hex_convert (char *);

/* Screen output routines: located in file SCRN_OUT.CPP
These include routines for outputing prompts: " _prompts"
and input fields: "_infields".  There is no separate screen
routine for the "address peek" menu item as this one only uses the
status window on the screen for input and output and all screen writing
is handled within the actual functional code module. */
	void paint_window (void);
	void clear_window (int, int);
	void compare_prompts (void);
	void program_prompts (void);
	void read_prompts (void);
	void menu_topics (void);
	void compare_infields (void);
	void program_infields (int);
	void show_device_selection (int, int);
	void read_infields (void);

/* Assembly language routine: source in T_LOOP.ASM; object file in *.OBJ */
	extern "C" {void timer (unsigned long);}


/* GLOBAL VARIABLE DECLARATIONS */

/* Programming and PROM read numerical variables */
	unsigned long start_byte, num_byte, start_addr, wrpulse_num, j_millisec;
	float final_pulse, pulse_len;
	int device_option = 1;

/* ASCII string versions of variables (as typed by user) */
	char in_fl_name[70], out_fl_name[70];
	char sstart_byte[20], snum_byte[20], sstart_addr[20];
	char swrpulse_num[20], spulse_len[20], sfinal_pulse[20];

/* Other internal use variables eg programmer control bytes */
	unsigned char bdata[16384], write_byte = 3;
	unsigned char verify_byte = 5, standby_a = 11, standby_b = 9;
	unsigned long twaste;

/* Screen color attribute variables (for color monitors) */
	int main_screen = 0x17;
	int menu_title = 0x1e;
	int active_prompts = 0x1f;
	int active_input = 0x7;
	int status_window = 0x71;
	int status_window_perm = 0x70;

/* Miscellaneous */
	int exit_code = 0;
	unsigned port_out, port_in, port_control;


int main (int argc, char *argv[])
{
	/* Local variable declarations */
	struct text_info t_info;
	char typed;
	/* Zero out first byte of global string arrays variables that are
	used for input by the user */
	in_fl_name[0] = 0;
	out_fl_name[0] = 0;
	sstart_byte[0] = 0;
	snum_byte[0] = 0;
	sstart_addr[0] = 0;
	swrpulse_num[0] = 0;
	spulse_len[0] = 0;
	sfinal_pulse[0] = 0;

	/* Process command line input, if any.  This allow input of non-
	standard parallel port addresses. First, set default addresses. */
	port_out = 0x378;
	port_in = 0x379;
	port_control = 0x37a;
	if (argc > 2)
	{
		cprintf ("Command line input error.\r\n");
		cprintf ("Correct format: EPROM x where x is optional and has the following");
		cprintf ("\r\nlegal values:  1 for parallel port base address 3BChex");
		cprintf ("\r\n2 for port address 278hex. Default is 378hex.");
		exit (0);
	}
	else;
	if (argc == 2)
	{
		switch (atoi(argv[1]))
		{
			case 1:
				port_out = 0x3BC;
				port_in = 0x3BD;
				port_control = 0x3BE;
				break;
			case 2:
				port_out = 0x278;
				port_in = 0x279;
				port_control = 0x27A;
				break;
			default:
				cprintf ("Command line input error.\r\n");
				cprintf ("Correct format: EPROM x where x is optional and has the following");
				cprintf ("\r\nlegal values:  1 for parallel port base address 3BChex");
				cprintf ("\r\n2 for port address 278hex. Default is 378hex.");
				exit (0);
		}
	}
	else;
	/* Check video mode and screen settings. Print error if non-
	supported mode is present on system. Set for direct video ram writing. */
	gettextinfo (&t_info);
	if (t_info.currmode >= 2)
		textmode (t_info.currmode);
	else
	{
		cprintf ("Error: Video mode (%d) that is not supported by this software", t_info.currmode);
		cprintf ("\r\nwas detected");
		cprintf ("\r\nExecution aborted.");
		exit (0);
	}
	_directvideo = 1;
	/* Set screen color variables to appropriate values if a monochrome
	monitor is being used */
	if ((t_info.currmode == 2) || (t_info.currmode == 7))
	{
		main_screen = 0x17;
		menu_title = 0x9;
		active_prompts = 0x1f;
		active_input = 0x70;
		status_window = 0x70;
		status_window_perm = 0x70;
	}
	else;
	/* Set programmer control lines to "EPROM read" mode */
	outportb (port_control, 0x5);
	/* Fetch values from EPROM.DFT file */
	get_dftfile ();
	/* Write background colour to whole screen */
	textattr (main_screen);
	clrscr ();
	/* Set up system timing */
	t_constant();
	/* Also calculate value of time wasting variable used in the "read_byte",
	"prom_write" and other PROM I/O routines. twaste has a value of
	50 microseconds minimum. */
	twaste = j_millisec/20;
	if (twaste < 10)
		twaste = 10;
	else;
	/* Paint start screen, using appropriate colors */
	paint_window ();
	textattr (menu_title);
	menu_topics ();
	textattr (main_screen);
	compare_prompts ();
	compare_infields ();
	program_prompts ();
	program_infields (main_screen);
	read_prompts ();
	read_infields ();
	/* Main loop -- continually repeated until an exit generated from
	within (alt-q pressed etc) */
	while (ALWAYS)
	{
	/* Check for ALT key sequence and action appropriate menu selection.
	Actioning involves highlighting appropriate prompts and input fields
	on the screen and printing message to status window for each prompt. */

	/* First, turn cursor off then get user input character and check if it
	is an ALT key sequence ie sent from keyboard as two bytes: a zero
	followed by an identifying	character.  If no leading zero, set the
	variable passed to the switch	statement to one that will cause a beep
	at the end.  Otherwise, check	and action. */
		_setcursortype (_NOCURSOR);
		typed = getch ();
		if (typed == 0)
			typed = getch ();
		else
			typed = 'x';
		switch (typed)
		{
		/* For the program, compare and read menu selections, the relevant
		case statement involves a common sequence:
			First print a message to the status window about bypassing data entry
			Then print prompts and their associated variables for the relevant
				menu item in highlight
			Fetch any user changes to these variables
			Redraw the input variables and call appropriate routine to
				read, write etc.
			When done, redraw the prompts and variables in normal (non-
				highlighted) colors
		After exit from any user input routine, there is a check to see
		whether the return resulted from the escape key being pressed (ie
		exit_code is set to 1).

		The Address Peek menu selection is much simpler as it only uses
		the status window for output. */

			/* If ALT-P, program menu item has been selected */
			case ALT_P:
			/* Output messages and prompts to screen */
			textattr (status_window);
			clear_window (status_window, 12);
			gotoxy (37, 12);
			cprintf ("Press ESC to continue without changes");
			textattr (active_prompts);
			compare_prompts ();
			program_prompts ();
			textattr (active_input);
			compare_infields ();
			program_infields (active_input);
			/* Fetch user input and if <ESC> was not pressed during input of the
			"compare" menu variables, also fetch the "programming" specific
			variables */
			getcompare_variables ();
			if (exit_code == 0)
				getprogram_variables (active_input);
			else;
			/* Call main writing routine in all cases -- this routines asks
			user whether they want to continue or abort. But first, re-output
			current variable values in case user began altering but decided
			to make no changes (ie pressed <ESC>). */
			textattr (active_input);
			compare_infields ();
			program_infields (active_input);
			prom_write ();
			/* Reset value of exit_code in preparation for next cycle through
			master loop and reset the screen output to the "inactive" state. */
			exit_code = 0;
			textattr (main_screen);
			compare_prompts ();
			program_prompts ();
			compare_infields ();
			program_infields (main_screen);
			break;

			/* If ALT-C, compare menu item has been selected */
			case ALT_C:
			/* Output messages and prompts to screen */
			textattr (status_window);
			clear_window (status_window, 12);
			gotoxy (37, 12);
			cprintf ("Press ESC to continue without changes");
			textattr (active_prompts);
			compare_prompts ();
			textattr (active_input);
			compare_infields ();
			getcompare_variables ();
			/* Call the actual "compare" routine. But first, re-output
			current variable values in case user began altering but decided
			to make no changes (ie pressed <ESC>). */
			textattr (active_input);
			compare_infields ();
			compare ();
			/* Reset value of exit_code in preparation for next cycle through
			master loop and reset the screen output to the "inactive" state. */
			exit_code = 0;
			textattr (main_screen);
			compare_prompts ();
			compare_infields ();
			break;

			/* If ALT-R, read menu item has been selected */
			case ALT_R:
			/* Output messages and prompts to screen */
			textattr (status_window);
			clear_window (status_window, 12);
			gotoxy (37, 12);
			cprintf ("Press ESC to continue without changes");
			textattr (active_prompts);
			read_prompts ();
			textattr (active_input);
			read_infields ();
			getread_variables ();
			/* Call routine to read PROM. But first, re-output
			current variable values in case user began altering but decided
			to make no changes (ie pressed <ESC>).*/
			textattr (active_input);
			read_infields ();
			read_tofile ();
			/* Reset value of exit_code in preparation for next cycle through
			master loop and reset the screen output to the "inactive" state. */
			exit_code = 0;
			textattr (main_screen);
			read_prompts ();
			read_infields ();
			break;

			/* If ALT-A, address peek menu item has been selected */
			case ALT_A:
			_setcursortype (_NORMALCURSOR);
			manual_read ();
			_setcursortype (_NOCURSOR);
			break;

			/* ALT-Q means exit */
			case ALT_Q:
			/* Restore screen color to white on black, clear screen, restore
			cursor and store variables from this session in file before
			exiting. */
			textattr (0x7);
			clrscr ();
			_setcursortype (_NORMALCURSOR);
			write_dftfile ();
			exit (0);
			break;

			/* Beep for all other keys */
			default:
			sound (600);
			delay (70);
			nosound ();
		}
	}
}
